import { HttpResponse, HttpHeaders, HttpErrorResponse, HttpEventType, HttpClient, HttpHandler, HTTP_INTERCEPTORS } from '@angular/common/http';
import * as i0 from '@angular/core';
import { InjectionToken, Injectable, Inject, Optional, NgModule, SkipSelf, Directive, Input, HostListener } from '@angular/core';
import { DOCUMENT } from '@angular/common';
import { ResponseType, getClient } from '@tauri-apps/api/http';
import { Observable, from } from 'rxjs';
import { switchMap, map } from 'rxjs/operators';
import { open, Command } from '@tauri-apps/api/shell';
import { invoke } from '@tauri-apps/api/tauri';
export { convertFileSrc, transformCallback } from '@tauri-apps/api/tauri';

const CLIENT_OPTIONS = new InjectionToken('client-options');
class TauriBackend {
    constructor(document, clientOptions) {
        this.document = document;
        this.clientOptions = clientOptions;
    }
    handle(req) {
        return new Observable((observer) => {
            // handle local requests (e.g. /assets/i18n/en.json)
            const url = req.url.startsWith('/') ? `${this.document.location.origin}${req.url}` : req.url;
            const tauriResponse = this.getClient().pipe(switchMap((client) => client.request({
                method: req.method.toUpperCase(),
                url: url,
                headers: this.convertHttpHeadersToTauriRecords(req.headers),
                body: req.body || undefined,
                query: this.convertHttpHeadersToTauriRecords(req.params),
                responseType: this.toTauriResponseType(req.responseType),
            })));
            tauriResponse.subscribe((response) => {
                if (response.ok) {
                    observer.next(new HttpResponse({
                        body: response.data,
                        headers: new HttpHeaders(response.headers),
                        status: response.status,
                        url: response.url || undefined,
                    }));
                    observer.complete();
                }
                else {
                    observer.error(new HttpErrorResponse({
                        error: response.data,
                        headers: new HttpHeaders(response.headers),
                        status: response.status,
                        url: response.url || undefined,
                    }));
                }
                observer.next({ type: HttpEventType.Sent });
            });
        });
    }
    /**
     * The Tauri http module requires a record instead a Object-Map hybrid
     * @param map Object which has nearly the same behavior as a Map
     * @returns Object with the same key-value pairs as the Object-Map hybrid
     */
    convertHttpHeadersToTauriRecords(map) {
        const record = {};
        map.keys().forEach((key) => {
            record[key] = map.get(key);
        });
        return record;
    }
    /**
     * Maps the string representation to a tauri-enum response type
     */
    toTauriResponseType(responseType) {
        if (responseType === 'json') {
            return ResponseType.JSON;
        }
        if (responseType === 'text') {
            return ResponseType.Text;
        }
        return ResponseType.Binary;
    }
    getClient() {
        if (this.client) {
            return this.client;
        }
        return from(getClient(this.clientOptions));
    }
}
TauriBackend.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriBackend, deps: [{ token: DOCUMENT }, { token: CLIENT_OPTIONS, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
TauriBackend.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriBackend });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriBackend, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: Document, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [CLIENT_OPTIONS]
                }] }]; } });
/**
 * got this piece of code from https://github.com/angular/angular/blob/13.1.x/packages/common/http/src/interceptor.ts
 * and modified it a tiny bit
 */
class HttpTauriInterceptorHandler {
    constructor(next, interceptor) {
        this.next = next;
        this.interceptor = interceptor;
    }
    handle(req) {
        return this.interceptor.intercept(req, this.next);
    }
}

/**
 * got this piece of code from https://github.com/angular/angular/blob/919f8ae4e8bd59efd3cb0cc375588b4e55126a3d/packages/common/http/src/module.ts#L54
 * angular didn't export it so I have to implement it myself again and modified it a little bit for my usecase
 */
const interceptingHandler = (backend, interceptors = []) => {
    if (!interceptors) {
        return backend;
    }
    return interceptors.reduceRight((next, interceptor) => new HttpTauriInterceptorHandler(next, interceptor), backend);
};
class HttpTauriModule {
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('HttpTauriModule is already loaded. Import it in the AppModule only');
        }
    }
    static forRoot(clientOptions) {
        return {
            ngModule: HttpTauriModule,
            providers: [
                {
                    provide: CLIENT_OPTIONS,
                    useValue: clientOptions,
                }
            ]
        };
    }
}
HttpTauriModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: HttpTauriModule, deps: [{ token: HttpTauriModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule });
HttpTauriModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: HttpTauriModule });
HttpTauriModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: HttpTauriModule, providers: [
        HttpClient,
        {
            provide: HttpHandler,
            useFactory: interceptingHandler,
            deps: [TauriBackend, [new Optional(), new Inject(HTTP_INTERCEPTORS)]],
        },
        TauriBackend,
    ] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: HttpTauriModule, decorators: [{
            type: NgModule,
            args: [{
                    providers: [
                        HttpClient,
                        {
                            provide: HttpHandler,
                            useFactory: interceptingHandler,
                            deps: [TauriBackend, [new Optional(), new Inject(HTTP_INTERCEPTORS)]],
                        },
                        TauriBackend,
                    ]
                }]
        }], ctorParameters: function () { return [{ type: HttpTauriModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }]; } });

class ShellService {
    open(path, openWith) {
        return from(open(path, openWith));
    }
}
ShellService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ShellService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
ShellService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ShellService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ShellService, decorators: [{
            type: Injectable
        }] });
const COMMAND_PROGRAMM_TOKEN = new InjectionToken('program');
const COMMAND_ARGS_TOKEN = new InjectionToken('args');
const COMMAND_OPTIONS_TOKEN = new InjectionToken('options');
class CommandService {
    constructor(program, args, options) {
        this.command = new Command(program, args, options);
    }
    spawn() {
        return from(this.command.spawn()).pipe(map((child) => new ChildService(child)));
    }
    execute() {
        return from(this.command.execute());
    }
}
CommandService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: CommandService, deps: [{ token: COMMAND_PROGRAMM_TOKEN }, { token: COMMAND_ARGS_TOKEN, optional: true }, { token: COMMAND_OPTIONS_TOKEN, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
CommandService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: CommandService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: CommandService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [COMMAND_PROGRAMM_TOKEN]
                }] }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [COMMAND_ARGS_TOKEN]
                }] }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [COMMAND_OPTIONS_TOKEN]
                }] }]; } });
class ChildService {
    constructor(child) {
        this.child = child;
    }
    write(data) {
        return from(this.child.write(data));
    }
    kill() {
        return from(this.child.kill());
    }
}

class OpenDirective {
    constructor(shellService) {
        this.shellService = shellService;
    }
    onClick(event) {
        event.preventDefault();
        const targetElement = event.target;
        if (targetElement instanceof HTMLAnchorElement) {
            this.shellService
                .open(targetElement.href, this.openWith)
                .subscribe(() => null, (err) => console.error(err));
        }
        else {
            throw Error('Element is no AnchorElement');
        }
    }
}
OpenDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: OpenDirective, deps: [{ token: ShellService }], target: i0.ɵɵFactoryTarget.Directive });
OpenDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.1.2", type: OpenDirective, selector: "[tauriOpen]", inputs: { openWith: "openWith" }, host: { listeners: { "click": "onClick($event)" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: OpenDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[tauriOpen]'
                }]
        }], ctorParameters: function () { return [{ type: ShellService }]; }, propDecorators: { openWith: [{
                type: Input,
                args: ['openWith']
            }], onClick: [{
                type: HostListener,
                args: ['click', ['$event']]
            }] } });

class ShellModule {
}
ShellModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ShellModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
ShellModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ShellModule, declarations: [OpenDirective], exports: [OpenDirective] });
ShellModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ShellModule, providers: [
        ShellService,
    ] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ShellModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        OpenDirective,
                    ],
                    providers: [
                        ShellService,
                    ],
                    exports: [
                        OpenDirective,
                    ]
                }]
        }] });

class TauriService {
    /**
     * This is a wrapper function.
     *
     * See {@link invoke} for official documentation
     */
    invoke(cmd, args) {
        return from(invoke(cmd, args));
    }
}
TauriService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
TauriService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriService, decorators: [{
            type: Injectable
        }] });

class TauriModule {
}
TauriModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
TauriModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriModule });
TauriModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriModule, providers: [
        TauriService,
    ] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: TauriModule, decorators: [{
            type: NgModule,
            args: [{
                    providers: [
                        TauriService,
                    ],
                }]
        }] });

/*
 * Public API Surface of ngx-tauri
 */

/**
 * Generated bundle index. Do not edit.
 */

export { HttpTauriModule, OpenDirective, ShellModule, TauriModule, TauriService };
//# sourceMappingURL=ngx-tauri.mjs.map
